@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 
-import json
+import sys
 import traceback
 from argparse import ArgumentParser
 import onnx
@@ -33,6 +33,33 @@ class Color:
     BG_DEFAULT     = '\033[49m'
     RESET          = '\033[0m'
 
+AVAILABLE_DTYPES = [
+    'float32',
+    'float64',
+    'int32',
+    'int64',
+    'str',
+]
+
+DTYPES_TO_ONNX_DTYPES = {
+    float: onnx.TensorProto.FLOAT,
+    int: onnx.TensorProto.INT64,
+    str: onnx.TensorProto.STRING,
+}
+
+DTYPES_TO_NUMPY_TYPES = {
+    'float32': np.float32,
+    'float64': np.float64,
+    'int32': np.int32,
+    'int64': np.int64,
+}
+
+NUMPY_TYPES_TO_ONNX_DTYPES = {
+    np.dtype('float32'): onnx.TensorProto.FLOAT,
+    np.dtype('float64'): onnx.TensorProto.DOUBLE,
+    np.dtype('int32'): onnx.TensorProto.INT32,
+    np.dtype('int64'): onnx.TensorProto.INT64,
+}
 
 def generate(
     op_type: str,
@@ -59,21 +86,21 @@ def generate(
     input_variables: Optional[dict]
         Specify input variables for the OP to be generated.\n\
         See below for the variables that can be specified.\n\n\
-        {'input_var_name1': [numpy.dtype, shape], 'input_var_name2': [dtype, shape], ...}\n\n\
-        e.g. input_variables = {'name1': [np.float32, [1,224,224,3]], 'name2': [np.bool_, [0]], ...}\n\
+        {"input_var_name1": [numpy.dtype, shape], "input_var_name2": [dtype, shape], ...}\n\n\
+        e.g. input_variables = {"name1": [np.float32, [1,224,224,3]], "name2": [np.bool_, [0]], ...}\n\
         https://github.com/onnx/onnx/blob/main/docs/Operators.md
 
     output_variables: Optional[dict]
         Specify output variables for the OP to be generated.\n\
         See below for the variables that can be specified.\n\n\
-        {'output_var_name1': [numpy.dtype, shape], 'output_var_name2': [dtype, shape], ...}\n\n\
-        e.g. output_variables = {'name1': [np.float32, [1,224,224,3]], 'name2': [np.bool_, [0]], ...}\n\
+        {"output_var_name1": [numpy.dtype, shape], "output_var_name2": [dtype, shape], ...}\n\n\
+        e.g. output_variables = {"name1": [np.float32, [1,224,224,3]], "name2": [np.bool_, [0]], ...}\n\
         https://github.com/onnx/onnx/blob/main/docs/Operators.md
 
     attributes: Optional[dict]
         Specify output attributes for the OP to be generated.\n\
         See below for the attributes that can be specified.\n\n\
-        {'attr_name1': value1, 'attr_name2': value2, 'attr_name3': value3, ...}\n\n\
+        {"attr_name1": value1, "attr_name2": value2, "attr_name3": value3, ...}\n\n\
         e.g. attributes = {"alpha": 1.0, "beta": 1.0, "transA": 0, "transB": 0}\n\
         Default: None\n\
         https://github.com/onnx/onnx/blob/main/docs/Operators.md
@@ -119,21 +146,63 @@ def generate(
         output_gs_variables = [gs.Variable(name=key, dtype=value[0], shape=value[1]) for key, value in output_variables.items()]
 
     # 2. Node Generation
-    node = gs.Node(
-        op=op_type,
-        attrs=attributes,
-        inputs=input_gs_variables,
-        outputs=output_gs_variables
-    )
+    node = None
+    value_info = None
+    if op_type not in ['Constant', 'ConstantOfShape']:
+        # non constant
+        node = gs.Node(
+            op=op_type,
+            attrs=attributes,
+            inputs=input_gs_variables,
+            outputs=output_gs_variables
+        )
+    else:
+        # constant
+        for attr_name, attr_values in attributes.items():
+
+            dtype = None
+            if isinstance(attr_values, np.ndarray):
+                dtype = NUMPY_TYPES_TO_ONNX_DTYPES[attr_values.dtype]
+            else:
+                dtype = DTYPES_TO_ONNX_DTYPES[type(attr_values)]
+
+            constant_name = [i for i in output_variables.keys()][0]
+            value_info = onnx.helper.make_tensor_value_info(
+                constant_name,
+                dtype,
+                attr_values.shape
+            )
+            node = onnx.helper.make_node(
+                op_type,
+                inputs=[],
+                outputs=[constant_name],
+                value=onnx.helper.make_tensor(
+                    name='value',
+                    data_type=dtype,
+                    dims=attr_values.shape,
+                    vals=attr_values,
+                ),
+            )
 
     # 3. Graph Generation
-    graph = gs.Graph(
-        nodes=[node],
-        inputs=input_gs_variables,
-        outputs=output_gs_variables,
-        opset=opset,
-    )
-    single_op_graph = gs.export_onnx(graph)
+    single_op_graph = None
+    if op_type not in ['Constant', 'ConstantOfShape']:
+        graph = gs.Graph(
+            nodes=[node],
+            inputs=input_gs_variables,
+            outputs=output_gs_variables,
+            opset=opset,
+        )
+        single_op_graph = gs.export_onnx(graph)
+    else:
+
+        graph_def = onnx.helper.make_graph(
+            nodes=[node],
+            name=op_type,
+            inputs=[],
+            outputs=[value_info],
+        )
+        single_op_graph = onnx.helper.make_model(graph_def)
 
     # 4. Graph Check
     try:
@@ -186,7 +255,7 @@ def main():
     parser.add_argument(
         '--input_variables',
         type=str,
-        nargs='+',
+        nargs=3,
         action='append',
         help=\
             'input_variables can be specified multiple times. \n'+
@@ -200,7 +269,7 @@ def main():
     parser.add_argument(
         '--output_variables',
         type=str,
-        nargs='+',
+        nargs=3,
         action='append',
         help=\
             'output_variables can be specified multiple times. \n'+
@@ -213,15 +282,18 @@ def main():
     )
     parser.add_argument(
         '--attributes',
-        type=json.loads,
+        nargs=3,
+        action='append',
         help=\
             'attributes can be specified multiple times. \n'+
-            'The key name is a string and the delimiter is double-cotation marks. \n'+
-            'Note that double-cotation marks must be escaped with a backslash. \n'+
-            '--attributes {"attribute_name1": value1, "attribute_name2": value2, ...} \n'+
+            '--attributes name dtype value \n'+
+            'dtype is one of "float32" or "float64" or "int32" or "int64" or "str". \n'+
             'https://github.com/onnx/onnx/blob/main/docs/Operators.md \n\n'+
             'e.g.\n'+
-            '--attributes "{\\"alpha\\": 1.0, \\"beta\\": 1.0, \\"transA\\": 0, \\"transB\\": 0}"'
+            '--attributes alpha float32 1.0 \n'+
+            '--attributes beta float32 1.0 \n'+
+            '--attributes transA int64 0 \n'+
+            '--attributes transB int64 0'
     )
     parser.add_argument(
         '--output_onnx_file_path',
@@ -254,6 +326,46 @@ def main():
     if args.output_variables:
         output_variables_tmp = {output_variable[0]: [getattr(np, output_variable[1]), eval(output_variable[2])] for output_variable in args.output_variables}
 
+    # attributes
+    """
+    attributes_tmp = {'name': value}
+    """
+    attributes_tmp = None
+    if args.attributes:
+
+        if args.op_type in ['Constant','ConstantOfShape']:
+            if len(args.attributes) > 1:
+                print(
+                    f'{Color.RED}ERROR:{Color.RESET} '+
+                    f'Only one attribute may be specified for Constant and ConstantOfShape.'
+                )
+                sys.exit(1)
+
+        attributes_tmp = {}
+        for attribute in args.attributes:
+            # parse
+            attr_name = attribute[0]
+            attr_type = attribute[1]
+            attr_value = eval(attribute[2])
+
+            # dtype check
+            if attr_type not in AVAILABLE_DTYPES:
+                print(
+                    f'{Color.RED}ERROR:{Color.RESET} '+
+                    f'The dtype that can be specified for attributes is one of the {AVAILABLE_DTYPES}. \n'+
+                    f'dtype:{attr_type}'
+                )
+                sys.exit(1)
+
+            # Conversion from python types to numpy types
+            # However, only if the input values are in list format
+            # Constant(value), ConstantOfShape
+            if (args.op_type == 'Constant' and attr_name in ['sparse_value', 'value']) or (args.op_type == 'ConstantOfShape'):
+                if isinstance(attr_value, list):
+                    attr_value = np.asarray(attr_value, dtype=DTYPES_TO_NUMPY_TYPES[attr_type])
+
+            attributes_tmp[attr_name] = attr_value
+
     # output_onnx_file_path
     output_onnx_file_path = ''
     if args.output_onnx_file_path:
@@ -267,7 +379,7 @@ def main():
         opset=args.opset,
         input_variables=input_variables_tmp,
         output_variables=output_variables_tmp,
-        attributes=args.attributes,
+        attributes=attributes_tmp,
         output_onnx_file_path=output_onnx_file_path,
         non_verbose=args.non_verbose,
     )
